==> pspresent-0.9.tar.gz: pspresent-0.9/ps.c <==
/*
   pspresent: PostScript presentation tool
   ps.c: Parsing of PostScript Document Structuring Convention
   Copyright (C) Matthew Chapman 2001-2002

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "pspresent.h"

#define DSC_END_COMMENTS   "%%EndComments"
#define DSC_BOUNDING_BOX   "%%BoundingBox: "
#define DSC_ORIENTATION    "%%Orientation: "
#define DSC_PAGES          "%%Pages: "
#define DSC_PAGE           "%%Page: "
#define DSC_BEGIN_DOCUMENT "%%BeginDocument"
#define DSC_END_DOCUMENT   "%%EndDocument"
#define DSC_TRAILER        "%%Trailer"
#define LEN(str)           (sizeof(str)-1)

static char *PSGetLine(char **p, char *end)
{
	char *next = *p;

	if (next != NULL)
	{
		*p = memchr(next, '\n', end-next);
		if (*p != NULL)
			(*p)++;
	}

	return next;
}

static Bool PSGetBoundingBox(char *string, int bounds[4])
{
	char *p;

	bounds[0] = strtol(string, &p, 10);
	if (*p != ' ')
		return False;

	bounds[1] = strtol(p+1, &p, 10);
	if (*p != ' ')
		return False;

	bounds[2] = strtol(p+1, &p, 10);
	if (*p != ' ')
		return False;

	bounds[3] = strtol(p+1, &p, 10);
	if ((*p != '\n') && (*p != 0))
		return False;

	return True;
}

static Bool PSGetOrientation(char *string, int *orientation)
{
	if (!strncmp(string, "Portrait", 8))
		*orientation = Portrait;
	else if (!strncmp(string, "Landscape", 9))
		*orientation = Landscape;
	else if (!strncmp(string, "Upside-Down", 11))
		*orientation = UpsideDown;
	else if (!strncmp(string, "Seascape", 8))
		*orientation = Seascape;
	else
		return False;

	return True;
}

Bool PSScanDocument(char *start, char *end, int bounds[4], int *orientation, int *num_pages, char ***page_offsets)
{
	char *line;
	Bool gotBB = False, gotOR = False;
	int pages = 0, page = 0;
	int eps_level = 0;
	char **offsets;

	while ((line = PSGetLine(&start, end)) != NULL)
	{
		if ((line[0] != '%')
		    || !strncmp(line, DSC_END_COMMENTS, LEN(DSC_END_COMMENTS)))
		{
			break;
		}
		else if (!strncmp(line, DSC_BOUNDING_BOX, LEN(DSC_BOUNDING_BOX)))
		{
			gotBB = PSGetBoundingBox(line + LEN(DSC_BOUNDING_BOX), bounds);
		}
		else if (!strncmp(line, DSC_ORIENTATION, LEN(DSC_ORIENTATION)))
		{
			gotOR = PSGetOrientation(line + LEN(DSC_ORIENTATION), orientation);
		}
		else if (!strncmp(line, DSC_PAGES, LEN(DSC_PAGES)))
		{
			pages = strtol(line + LEN(DSC_PAGES), NULL, 10) + 1;
		}
	}

	if (!gotBB)
		return False;

	if (!gotOR)
		*orientation = Portrait;

	if (pages == 0)
		pages = 100;

	offsets = malloc(pages * sizeof(char *));

	while ((line = PSGetLine(&start, end)) != NULL)
	{
		if (line[0] != '%')
		{
		}
		else if (!strncmp(line, DSC_BEGIN_DOCUMENT, LEN(DSC_BEGIN_DOCUMENT)))
		{
			eps_level++;
		}
		else if (!strncmp(line, DSC_END_DOCUMENT, LEN(DSC_END_DOCUMENT)))
		{
			eps_level--;
		}
		else if (!strncmp(line, DSC_PAGE, LEN(DSC_PAGE)) && (eps_level == 0))
		{
			offsets[page++] = line;
			if (page == pages)
			{
				pages += 100;
				offsets = realloc(offsets, pages * sizeof(char *));
			}
		}
		else if (!strncmp(line, DSC_TRAILER, LEN(DSC_TRAILER)) && (eps_level == 0))
		{
			offsets[page] = line;
			break;
		}
	}

	if (page == 0)
	{
		free(offsets);
		return False;
	}

	*num_pages = page;
	*page_offsets = offsets;
	return True;
}

==> pspresent_1.0.orig.tar.gz: pspresent-1.0/ps.c <==
/*
   pspresent: PostScript presentation tool
   ps.c: Parsing of PostScript Document Structuring Convention
   Copyright (C) Matthew Chapman 2001-2002

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "pspresent.h"

#include <assert.h>

#define DSC_END_COMMENTS   "%%EndComments"
#define DSC_BOUNDING_BOX   "%%BoundingBox: "
#define DSC_ORIENTATION    "%%Orientation: "
#define DSC_PAGES          "%%Pages: "
#define DSC_PAGE           "%%Page: "
#define DSC_BEGIN_DOCUMENT "%%BeginDocument"
#define DSC_END_DOCUMENT   "%%EndDocument"
#define DSC_TRAILER        "%%Trailer"
#define LEN(str)           (sizeof(str)-1)

static char *PSGetLine(char **p, char *end)
{
	char *next = *p;

	if (next != NULL)
	{
		*p = memchr(next, '\n', end-next);
		if (*p != NULL)
			(*p)++;
	}

	return next;
}

static Bool PSGetBoundingBox(char *string, int bounds[4])
{
	char *p;

	bounds[0] = strtol(string, &p, 10);
	if (*p != ' ')
		return False;

	bounds[1] = strtol(p+1, &p, 10);
	if (*p != ' ')
		return False;

	bounds[2] = strtol(p+1, &p, 10);
	if (*p != ' ')
		return False;

	bounds[3] = strtol(p+1, &p, 10);
	if ((*p != '\n') && (*p != 0))
		return False;

	return True;
}

Bool PSGetOrientation(char *string, int *orientation)
{
	if (!strncmp(string, "Portrait", 8))
		*orientation = Portrait;
	else if (!strncmp(string, "Landscape", 9))
		*orientation = Landscape;
	else if (!strncmp(string, "Upside-Down", 11))
		*orientation = UpsideDown;
	else if (!strncmp(string, "Seascape", 8))
		*orientation = Seascape;
	else
		return False;

	return True;
}

Bool PSScanDocument(char *start, char *end, int bounds[4], int *orientation, 
		    int *num_pages, struct pspage **page_offsets,
		    int *num_real_pages, int **real_page_offsets)
{
	char *line;
	char *last_page = NULL;
	int last_len = 0;
	Bool gotBB = False, gotOR = False;
	int pages = 0, page = 0, non_overlay_page = 0;
	int eps_level = 0;
	struct pspage *offsets;
	int *real_offsets;

	while ((line = PSGetLine(&start, end)) != NULL)
	{
		if ((line[0] != '%')
		    || !strncmp(line, DSC_END_COMMENTS, LEN(DSC_END_COMMENTS)))
		{
			break;
		}
		else if (!strncmp(line, DSC_BOUNDING_BOX, LEN(DSC_BOUNDING_BOX)))
		{
			gotBB = PSGetBoundingBox(line + LEN(DSC_BOUNDING_BOX), bounds);
		}
		else if (!strncmp(line, DSC_ORIENTATION, LEN(DSC_ORIENTATION)))
		{
			gotOR = PSGetOrientation(line + LEN(DSC_ORIENTATION), orientation);
		}
		else if (!strncmp(line, DSC_PAGES, LEN(DSC_PAGES)))
		{
			pages = strtol(line + LEN(DSC_PAGES), NULL, 10) + 1;
		}
	}

	if (!gotBB)
		return False;

	if (!gotOR)
		*orientation = Portrait;

	if (pages == 0)
		pages = 100;

	offsets = malloc(pages * sizeof(struct pspage));
	real_offsets = malloc(pages * sizeof(int));

	while ((line = PSGetLine(&start, end)) != NULL)
	{
		if (line[0] != '%')
		{
		}
		else if (!strncmp(line, DSC_BEGIN_DOCUMENT, LEN(DSC_BEGIN_DOCUMENT)))
		{
			eps_level++;
		}
		else if (!strncmp(line, DSC_END_DOCUMENT, LEN(DSC_END_DOCUMENT)))
		{
			eps_level--;
		}
		else if (!strncmp(line, DSC_PAGE, LEN(DSC_PAGE)) && (eps_level == 0))
		{
			int len;
			len = strchr(LEN(DSC_PAGE) + line, ' ') - 
				(line + LEN(DSC_PAGE));

			if (last_page != NULL) {
				if (strncmp(LEN(DSC_PAGE) + line, 
					    last_page, last_len)) {
					offsets[page-1].is_last = 1;
					real_offsets[non_overlay_page++] = page - 1;
				} 
			}
			last_page = LEN(DSC_PAGE) + line;
			last_len = len;
			offsets[page].is_last = 0;
			offsets[page++].page = line;
			if (page == pages)
			{
				pages += 100;
				offsets = realloc(offsets, pages * sizeof(char *));
			}
		}
		else if (!strncmp(line, DSC_TRAILER, LEN(DSC_TRAILER)) && 
			 (eps_level == 0))
		{
			offsets[page].page = line;
			offsets[page-1].is_last = 1;
			real_offsets[non_overlay_page++] = page - 1;
			break;
		}
	}

	if (page == 0)
	{
		free(offsets);
		return False;
	}

	*num_pages = page;
	*page_offsets = offsets;
	*num_real_pages = non_overlay_page;
	*real_page_offsets = real_offsets;
	return True;
}
